using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MccDaq;

namespace AInScanForeground
{
    public partial class Form1 : Form
    {
        public MccDaq.MccBoard DaqBoard; 
        public MccDaq.ErrorInfo ULStat;
        public MccDaq.Range Range;
        public System.IntPtr AIMhandle;
        public System.Int32 AINumPoints;
        public System.Int32 numchannels;
        public System.Boolean Status;
        public System.Int32 AIFirstPoint, numpoints;
        public System.Int32 ChannelCount;
        public System.Int32 LowChannelNumber = 0;
        public System.Int32 HighChannelNumber = 0;
        System.Int32 Rate;
        System.Double[] ADDataV;
        System.Double[,] twoDData;
        System.Int32 m2DRows, rows, cols, arrayindex;

        BERGtools.Oscilloscope scope;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //First Lets make sure there's a USB-2408 of some kind plugged in,
            //then we'll check to see if it has analog outputs.
            System.Int16 BoardNum;
            System.Boolean Boardfound = false;
            for (BoardNum = 0; BoardNum < 99; BoardNum++)
            {

                DaqBoard = new MccDaq.MccBoard(BoardNum);
                if (DaqBoard.BoardName.Contains("2408-2AO"))
                {
                    Boardfound = true;
                    this.Text = "2408-2AO found as board number: " + BoardNum.ToString();
                    ULStat = DaqBoard.FlashLED();
                    break;
                }

                if (DaqBoard.BoardName.Contains("2408"))
                {
                    Boardfound = true;
                    this.Text = "USB-2408 found as board number: " + BoardNum.ToString();
                    DaqBoard.FlashLED();
                    break;
                }
            }

            if (Boardfound == false)
            {
                System.Windows.Forms.MessageBox.Show("No USB-2408 series board found in system.  Please run InstaCal.", "No Board detected");
                this.Close();
            }

            //Initialize objects on the form needing attention
            LoadComboBox(cmboAInRange);


            scope = new BERGtools.Oscilloscope();
            scope.Initialize(pbScope);                      //select the picturebox to use as stripchart
            scope.TraceColor(0,Color.White);                  //select the color of the data trace for channel 0
            scope.TraceColor(1, Color.Blue);                   //select the color of the data trace for channel 1
            scope.FontColor(System.Drawing.Color.Beige);  //select the color of the text
            scope.BackgroundColor(Color.DarkGreen);       //select the background color
            scope.ScaleColor(Color.DarkKhaki);               //select scale and tick mark color
            scope.YAxisScale(BERGtools.YAxisScaleVal.Bipolar);  //select if scale is uni or bi polar
            scope.YScale(-100, 100, 8);                       //enter the min, max, and number of ticks for the Y Scale
            scope.DrawBackground();                         //draw the bitmap into the picture box as background
        }

        public void LoadComboBox(ComboBox sender)
        {
            sender.Items.Add("BIP10VOLTS");
            sender.Items.Add("BIP5VOLTS");
            sender.Items.Add("BIP2PT5VOLTS");
            sender.Items.Add("BIP1PT25VOLTS");
            sender.Items.Add("BIPPT625VOLTS");
            sender.Items.Add("BIPPT312VOLTS");
            sender.Items.Add("BIPPT156VOLTS");
            sender.Items.Add("BIPPT078VOLTS"); 
            sender.SelectedIndex = 0;
        }

        public void SelectRange(Int32 ComboBoxIndex)
        {
            switch (ComboBoxIndex)
            {
                case 0:
                    Range = MccDaq.Range.Bip10Volts;
                    break;
                case 1:
                    Range = MccDaq.Range.Bip5Volts;
                    break;
                case 2:
                    Range = MccDaq.Range.Bip2Pt5Volts;
                    break;
                case 3:
                    Range = MccDaq.Range.Bip1Pt25Volts;
                    break;
                case 4:
                    Range = MccDaq.Range.BipPt625Volts;
                    break;
                case 5:
                    Range = MccDaq.Range.BipPt312Volts;
                    break;
                case 6:
                    Range = MccDaq.Range.BipPt156Volts;
                    break;
                case 7:
                    Range = MccDaq.Range.BipPt078Volts;
                    break;
            }
        }

        private void btnStartStop_Click(object sender, EventArgs e)
        {
             if (btnStartStop.Text == "Start") {
                btnStartStop.Text = "Stop";
                Status = true;
                this.Refresh();
                RunAdc();
			 }
			 else{
                btnStartStop.Text = "Start";
                Status = false;
                ULStat = DaqBoard.StopBackground(MccDaq.FunctionType.AiFunction);
                if (ULStat.Value !=  MccDaq.ErrorInfo.ErrorCode.NoErrors) this.Close();
                lblMode.Text = "Idle";
			    ULStat = MccDaq.MccService.WinBufFreeEx(AIMhandle); // Free up memory for use by other programs
                if (ULStat.Value != MccDaq.ErrorInfo.ErrorCode.NoErrors) this.Close();
			 }
        }

        public void RunAdc()
        {
		    MccDaq.ScanOptions AInOptions;
		    System.Int32 ChannelCount;

            //set low channel number
		    try
            {
                 Int32.TryParse(txtLowChannel.Text, out LowChannelNumber);
		    }
		    catch(Exception exc)
            {
                System.Windows.Forms.MessageBox.Show(txtLowChannel.Text + " is not a valid Low Channel Number value", 
			    "Invalid Channel Number " );
			    return;
		    }

            //set high channel number
		    try
            {
                Int32.TryParse(txtHighChannel.Text, out HighChannelNumber);
		    }
            catch (Exception exc)
            {
                System.Windows.Forms.MessageBox.Show(txtHighChannel.Text + " is not a valid High Channel Number value", 
			    "Invalid Channel Number ");
		    }

            //set up sample count and redim arrays
            try
            {
                Int32.TryParse(txtNumSamples.Text, out AINumPoints);
                numpoints = AINumPoints;
                ChannelCount = HighChannelNumber - LowChannelNumber + 1;
                AINumPoints = AINumPoints * ChannelCount;
                ADDataV = new Double[(ChannelCount) * (numpoints)];
                twoDData = new Double[(ChannelCount), (numpoints / ChannelCount)];
            }
            catch (Exception exc)
            {
                System.Windows.Forms.MessageBox.Show(txtNumSamples.Text + " is not a valid number of points value", 
                "Invalid Rate Number ", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            
            // set aside memory to hold data
            AIMhandle = MccDaq.MccService.ScaledWinBufAllocEx(AINumPoints);
            if (AIMhandle == IntPtr.Zero)
            {
		    System.Windows.Forms.MessageBox.Show("Bad Handle", "Invalid Handle");
		    return;
		    }

            AIFirstPoint = 0;

            //Set scan options
		    AInOptions = MccDaq.ScanOptions.ScaleData;

            //read in the selected range
            SelectRange(cmboAInRange.SelectedIndex);
            
            //read in the set sampling rate
            Int32.TryParse(txtRate.Text, out Rate);
            //Start the scan
            lblMode.Text = "Running";
            this.Refresh();
            ULStat = DaqBoard.AInScan(LowChannelNumber, HighChannelNumber, AINumPoints,
                ref Rate, Range, AIMhandle, AInOptions);
		    if (ULStat.Value != MccDaq.ErrorInfo.ErrorCode.NoErrors) 
            {
                errhandler(ULStat);
                return;
			}

            //txtRate.Text = Rate.ToString();

            ConvertData();

		    btnStartStop.Text = "Start";
            ULStat = DaqBoard.StopBackground(MccDaq.FunctionType.AiFunction);
            lblMode.Text = "Idle";
            //this.Refresh();
            ULStat = MccDaq.MccService.WinBufFreeEx(AIMhandle); // Free up memory for use by other programs
		    if (ULStat.Value != MccDaq.ErrorInfo.ErrorCode.NoErrors ){
                errhandler(ULStat);
                return;
		    }
	    }

        public void errhandler(MccDaq.ErrorInfo ULStat )
        {
            //Generic UL error handler
	        System.Windows.Forms.MessageBox.Show(ULStat.Message, "Universal Library Error " );
            lblMode.Text = "Idle";
            btnStartStop.Text = "Start";
            ULStat = DaqBoard.StopBackground(MccDaq.FunctionType.AiFunction);
            ULStat = MccDaq.MccService.WinBufFreeEx(AIMhandle); // Free up memory for use by other programs
        }

        public void ConvertData(){
            /*Convert the data from counts to volts, then finally do something useful with the data.
            Here the data is plotted in a picture box as a graph.*/
            lblMode.Text = "Processing"; 
		    ChannelCount = Convert.ToInt32(txtHighChannel.Text) - Convert.ToInt32(txtLowChannel.Text) + 1;

            ULStat = MccDaq.MccService.ScaledWinBufToArray(AIMhandle, ADDataV, AIFirstPoint, numpoints); 

            for (cols = 0; cols < ChannelCount; cols++)
            {
                m2DRows = -1;
                for (rows = 0; rows < numpoints; rows += ChannelCount)
                {
                    arrayindex = cols + rows;
                    m2DRows += 1;
                    if (m2DRows < numpoints/ChannelCount)
                       twoDData[cols, m2DRows] = ADDataV[arrayindex];
                }
            }
            scope.UpdateScope(ref twoDData);   //twoDData is an array of new data to send to the oscilloscope
        }

        private void btnEnd_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}