using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MccDaq;
using BERGtools;

namespace AInScanBackgroundContinuous
{
    public partial class Form1 : Form
    {
        public MccDaq.MccBoard DaqBoard;
        public MccDaq.ErrorInfo ULStat;
        public MccDaq.Range Range;
        System.IntPtr AIMhandle;
        System.Int32 AINumPoints;
        System.Int32 numchannels, MidScanPoint, numpoints;
        System.Int32 ADRate;
        System.Double[] ADDataV;
        System.Double[,] twoDData;
        short Status;
        System.Int32 m2DRows, rows, cols, arrayindex;
        System.Int32 CurCount, CurIndex;
        System.Int32 ChannelCount;
        System.Int32 LowChannelNumber, HighChannelNumber;
        System.String LastPass = "Upper";

        BERGtools.Oscilloscope scope;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //First Lets make sure there's a USB-2408 of some kind plugged in,
            //then we'll check to see if it has analog outputs.
            System.Int16 BoardNum;
            System.Boolean Boardfound = false;
            for (BoardNum = 0; BoardNum < 99; BoardNum++)
            {

                DaqBoard = new MccDaq.MccBoard(BoardNum);
                if (DaqBoard.BoardName.Contains("2408-2AO"))
                {
                    Boardfound = true;
                    this.Text = "2408-2AO found as board number: " + BoardNum.ToString();
                    ULStat = DaqBoard.FlashLED();
                    break;
                }

                if (DaqBoard.BoardName.Contains("2408"))
                {
                    Boardfound = true;
                    this.Text = "USB-2408 found as board number: " + BoardNum.ToString();
                    DaqBoard.FlashLED();
                    break;
                }
            }

            if (Boardfound == false)
            {
                System.Windows.Forms.MessageBox.Show("No USB-2408 series board found in system.  Please run InstaCal.", "No Board detected");
                this.Close();
            }

            //Initialize objects on the form as needed
            LoadComboBox(cmboAInRange);

            //Determine if the device is set for single ended or differential by the number of channels.
            DaqBoard.BoardConfig.GetNumAdChans(out numchannels);
            nudLowChannel.Maximum = numchannels;
            nudHighChannel.Maximum = numchannels;


            scope = new BERGtools.Oscilloscope();
            scope.Initialize(pbScope);                      //select the picturebox to use as stripchart
            scope.TraceColor(0,Color.White);                  //select the color of the data trace for channel 0
            scope.TraceColor(1, Color.Blue);                   //select the color of the data trace for channel 1
            scope.FontColor(System.Drawing.Color.Beige);  //select the color of the text
            scope.BackgroundColor(Color.DarkGreen);       //select the background color
            scope.ScaleColor(Color.DarkKhaki);               //select scale and tick mark color
            scope.YAxisScale(BERGtools.YAxisScaleVal.Bipolar);  //select if scale is uni or bi polar
            scope.YScale(-100, 100, 10);                       //enter the min, max, and number of ticks for the Y Scale
            scope.DrawBackground();                         //draw the bitmap into the picture box as background
        }

        public void LoadComboBox(ComboBox sender)
        {
            sender.Items.Add("BIP10VOLTS");
            sender.Items.Add("BIP5VOLTS");
            sender.Items.Add("BIP2PT5VOLTS");
            sender.Items.Add("BIP1PT25VOLTS");
            sender.Items.Add("BIPPT625VOLTS");
            sender.Items.Add("BIPPT312VOLTS");
            sender.Items.Add("BIPPT156VOLTS");
            sender.Items.Add("BIPPT078VOLTS"); 
            sender.SelectedIndex = 0;
        }

        public void SelectRange(Int32 ComboBoxIndex)
        {
            switch (ComboBoxIndex)
            {
                case 0:
                    Range = MccDaq.Range.Bip10Volts;
                    break;
                case 1:
                    Range = MccDaq.Range.Bip5Volts;
                    break;
                case 2:
                    Range = MccDaq.Range.Bip2Pt5Volts;
                    break;
                case 3:
                    Range = MccDaq.Range.Bip1Pt25Volts;
                    break;
                case 4:
                    Range = MccDaq.Range.BipPt625Volts;
                    break;
                case 5:
                    Range = MccDaq.Range.BipPt312Volts;
                    break;
                case 6:
                    Range = MccDaq.Range.BipPt156Volts;
                    break;
                case 7:
                    Range = MccDaq.Range.BipPt078Volts;
                    break;
            }

        }

        private void btnEnd_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnAInStartStop_Click(object sender, EventArgs e)
        {
            if (btnAInStartStop.Text == "Start")
            {
                btnAInStartStop.Text = "Stop";
                this.Refresh();
                RunAdc();
            }
            else
            {
                btnAInStartStop.Text = "Start";
                tmrAInScan.Enabled = false;
                ULStat = DaqBoard.StopBackground(MccDaq.FunctionType.AiFunction);
                if (ULStat.Value != MccDaq.ErrorInfo.ErrorCode.NoErrors) this.Close();
                lblMode.Text = "Idle";
                ULStat = MccDaq.MccService.WinBufFreeEx(AIMhandle); // Free up memory for use by other programs
                if (ULStat.Value != MccDaq.ErrorInfo.ErrorCode.NoErrors) this.Close();
            }
        }

        public void errhandler(MccDaq.ErrorInfo ULStat)
        {
            //Generic UL error handler
            System.Windows.Forms.MessageBox.Show(ULStat.Message, "Universal Library Error ");
            lblMode.Text = "Idle";
            btnAInStartStop.Text = "Start";
            ULStat = DaqBoard.StopBackground(MccDaq.FunctionType.AiFunction);
            ULStat = MccDaq.MccService.WinBufFreeEx(AIMhandle); // Free up memory for use by other programs
        }

        public void RunAdc()
        {
            MccDaq.ScanOptions AInOptions;

            //Read in the Low and High channel settings, and calculate the number of channels to scan
            LowChannelNumber = Convert.ToInt32(nudLowChannel.Value);

            HighChannelNumber = Convert.ToInt32(nudHighChannel.Value);

            

            //read in the sampling rate/channel
            try
            {
                Int32.TryParse(txtRate.Text, out ADRate);
            }
            catch (Exception exc)
            {
                MessageBox.Show(txtRate.Text + " is not a valid rate value", "Invalid count ", 
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                btnAInStartStop.Text = "Start";
                return;
            }

            //read in the number of samples/channel
            try
            {
                Int32.TryParse(txtNumSamples.Text, out AINumPoints);
            }
            catch (Exception exc)
            {
                MessageBox.Show(txtNumSamples.Text + " is not a valid number of points value", "Invalid Rate Number ", 
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                btnAInStartStop.Text = "Start";
                return;
            }

            //adjust the arrays to handle the correct amount of samples
            ChannelCount = HighChannelNumber - LowChannelNumber + 1;
            numpoints = AINumPoints;
            AINumPoints = AINumPoints * ChannelCount * 2;
            MidScanPoint = Convert.ToInt32(AINumPoints / ChannelCount / 2);
            ADDataV = new Double[(ChannelCount) * (numpoints)];
            twoDData = new Double[(ChannelCount), (numpoints/ ChannelCount)];



            // set aside memory to hold data (create a handle)
            AIMhandle = MccDaq.MccService.ScaledWinBufAllocEx(AINumPoints);
            if (AIMhandle == IntPtr.Zero)
            {
                System.Windows.Forms.MessageBox.Show("Bad Handle", "Invalid Handle");
                return;
            }

            //Set scan options
            AInOptions = ScanOptions.Background | ScanOptions.ScaleData;
            if (chkContinuous.Checked)
                AInOptions = AInOptions | ScanOptions.Continuous;

            //Get the selected A/D range
            SelectRange(cmboAInRange.SelectedIndex);

            //Start the Scan
            lblMode.Text = "Running";
            ULStat = DaqBoard.AInScan(LowChannelNumber, HighChannelNumber, AINumPoints, ref  ADRate, Range, AIMhandle, AInOptions);
            if (ULStat.Value != MccDaq.ErrorInfo.ErrorCode.NoErrors)
            {
                errhandler(ULStat);
                return;
            }

            //txtRate.Text = ADRate.ToString();

            tmrAInScan.Enabled = true;
        }

        private void tmrAInScan_Tick(object sender, EventArgs e)
        {
            ULStat = DaqBoard.GetStatus(out Status, out CurCount, out CurIndex, MccDaq.FunctionType.AiFunction);
            if (ULStat.Value != MccDaq.ErrorInfo.ErrorCode.NoErrors)
            {
                errhandler(ULStat);
                return;
            }

            lblCurCount.Text = CurCount.ToString();
            lblCurIndex.Text = CurIndex.ToString();
            if (Status == MccDaq.MccBoard.Running)
            {

                lblMode.Text = "Running";

                /* at every timer tick, determine if there is new data available. */
                ConvertData();

            }
            if (Status == MccDaq.MccBoard.Idle)
            {
                tmrAInScan.Enabled = false;
                btnAInStartStop.Text = "Start";
                ULStat = DaqBoard.StopBackground(MccDaq.FunctionType.AiFunction);
                lblMode.Text = "Idle";
                ULStat = MccDaq.MccService.WinBufFreeEx(AIMhandle); // Free up memory for use by other programs
                if (ULStat.Value != MccDaq.ErrorInfo.ErrorCode.NoErrors)
                {
                    errhandler(ULStat);
                    return;
                }
            }
        }

        public void ConvertData()
        {
            /*Read the data from the buffer, then do something useful with the data.
            'Here the data is plotted in a picture box as a graph.*/
             ULStat = DaqBoard.GetStatus(out Status, out CurCount, out CurIndex, MccDaq.FunctionType.AiFunction);
			if (ULStat.Value != MccDaq.ErrorInfo.ErrorCode.NoErrors ){
				errhandler(ULStat);
				return;
			}

			//get data from the buffer, 1/2 at a time....
            if (CurIndex > MidScanPoint)
            {      //get the lower half of the buffer
				if (AIMhandle == IntPtr.Zero) return;
                if ((LastPass == "Lower") & (Status == MccDaq.MccBoard.Running))  return;
                LastPass = "Lower";
                ULStat = MccDaq.MccService.ScaledWinBufToArray(AIMhandle, ADDataV, 0, MidScanPoint - 1);
				if (ULStat.Value != MccDaq.ErrorInfo.ErrorCode.NoErrors ){
					errhandler(ULStat);
					return;
				}
			}

			else  {        //get upper half of buffer
                if ((LastPass == "Upper") & (Status == MccDaq.MccBoard.Running)) return;
                LastPass = "Upper";
                ULStat = MccDaq.MccService.ScaledWinBufToArray(AIMhandle, ADDataV, MidScanPoint, MidScanPoint);
				if (ULStat.Value != MccDaq.ErrorInfo.ErrorCode.NoErrors ){
					errhandler(ULStat);
					return;
				}

			 }

            //convert the data from a 1D array to a 2D array so it can be plotted
            for (cols = 0; cols < ChannelCount; cols++)
            {
                m2DRows = -1;
                for ( rows = 0; rows < numpoints; rows += ChannelCount)
                {
                    arrayindex = cols + rows;
                    m2DRows += 1;
                    twoDData[cols, m2DRows] = ADDataV[arrayindex];
                }
            }

            scope.UpdateScope(ref twoDData);   //twoDData is an array of new data to send to the oscilloscope class.
        }
    }
}