﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using MccDaq;

namespace AnalogIn_toStripchart_SW_Timed
{
    public partial class Form1 : Form
    {

        public MccDaq.MccBoard DaqBoard;
        public MccDaq.ErrorInfo ULStat;
        public MccDaq.Range Range;
        public System.Int32 numchannels = 0;

        BERGtools.StripChart Charter;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //First Lets make sure there's a USB-2408 of some kind plugged in,
            //then we'll check to see if it has analog outputs.
            System.Int16 BoardNum;
            System.Boolean Boardfound = false;
            for (BoardNum = 0; BoardNum < 99; BoardNum++)
            {
                DaqBoard = new MccDaq.MccBoard(BoardNum);
                if (DaqBoard.BoardName.Contains("2408-2AO"))
                {
                    Boardfound = true;
                    this.Text = "2408-2AO found as board number: " + BoardNum.ToString();
                    ULStat = DaqBoard.FlashLED();
                    break;
                }

                if (DaqBoard.BoardName.Contains("2408"))
                {
                    Boardfound = true;
                    this.Text = "USB-2408 found as board number: " + BoardNum.ToString();
                    DaqBoard.FlashLED();
                    break;
                }
            }

            if (Boardfound == false)
            {
                System.Windows.Forms.MessageBox.Show("No USB-2408 series board found in system.  Please run InstaCal.", "No Board detected");
                this.Close();
            }

            //Initialize objects on the form needing attention
            LoadComboBox(cmboAInRange);

            //Determine if the device is set for single ended or differential by the number of channels.
            DaqBoard.BoardConfig.GetNumAdChans(out numchannels);
            nudAInChannel.Maximum = numchannels;

            Charter = new BERGtools.StripChart();
            Charter.Initialize(pbStripChart);                        //select the picturebox to use as stripchart
            Charter.FontColor(Color.Beige);         //select the color of the text
            Charter.BackgroundColor(Color.Blue);                   //select the background color
            Charter.SelectPenColor(Color.SkyBlue);                   // select data trace color
            Charter.ScaleColor(Color.Yellow);                        //select scale and tick mark color
            Charter.YScale(-10, 10, 10, true);                           //enter the min, max, and number of ticks for the Y Scale
            Charter.XScale(tmrAnalogIn.Interval, 3);                //enter the timer interval and number of ticks for X scale
            Charter.DrawBackground();

            //set up sample timing
            for (int i = 1; i < 11; i++)
                cbRate.Items.Add(i);
            cbRate.SelectedIndex = 9;

        }
        public void LoadComboBox(ComboBox sender)
        {
            sender.Items.Add("BIP10VOLTS");
            sender.Items.Add("BIP5VOLTS");
            sender.Items.Add("BIP2PT5VOLTS");
            sender.Items.Add("BIP1PT25VOLTS");
            sender.Items.Add("BIPPT625VOLTS");
            sender.Items.Add("BIPPT312VOLTS");
            sender.Items.Add("BIPPT156VOLTS");
            sender.Items.Add("BIPPT078VOLTS");
            sender.SelectedIndex = 0;
        }

        public void SelectRange(Int32 ComboBoxIndex)
        {
            switch (ComboBoxIndex)
            {
                case 0:
                    Range = MccDaq.Range.Bip10Volts;
                    break;
                case 1:
                    Range = MccDaq.Range.Bip5Volts;
                    break;
                case 2:
                    Range = MccDaq.Range.Bip2Pt5Volts;
                    break;
                case 3:
                    Range = MccDaq.Range.Bip1Pt25Volts;
                    break;
                case 4:
                    Range = MccDaq.Range.BipPt625Volts;
                    break;
                case 5:
                    Range = MccDaq.Range.BipPt312Volts;
                    break;
                case 6:
                    Range = MccDaq.Range.BipPt156Volts;
                    break;
                case 7:
                    Range = MccDaq.Range.BipPt078Volts;
                    break;
            }
        }
        public void errhandler(MccDaq.ErrorInfo ULStat)
        {
            //Generic UL error handler
            System.Windows.Forms.MessageBox.Show(ULStat.Message, "Universal Library Error ");
            lblAIMode.Text = "Idle";
            btnStartStop.Text = "Start";
        }

        private void btnStartStop_Click(object sender, EventArgs e)
        {
            if (btnStartStop.Text == "Start")
            {
                btnStartStop.Text = "Stop";
                lblAIMode.Text = "Running";
                btnStartStop.Refresh();
                tmrAnalogIn.Interval = 1000 / Convert.ToInt32(cbRate.SelectedItem);
                tmrAnalogIn.Enabled = true;
            }
            else
            {
                btnStartStop.Text = "Start";
                lblAIMode.Text = "Idle";
                tmrAnalogIn.Enabled = false;
            }
        }

        private void Button6_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void tmrAnalogIn_Tick(object sender, EventArgs e)
        {
            System.Double VInVolts;

            SelectRange(cmboAInRange.SelectedIndex);

            ULStat = DaqBoard.VIn32(Convert.ToInt16(nudAInChannel.Value), Range, out VInVolts, MccDaq.VInOptions.Default);
            if (ULStat.Value != MccDaq.ErrorInfo.ErrorCode.NoErrors)
            {
                errhandler(ULStat);
                return;
            }

            txtADValue.Text = VInVolts.ToString();
            Charter.DrawStripChart(ref VInVolts);
        }

    }
}
