using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MccDaq;

namespace AnalogOut
{
    public partial class Form1 : Form
    {
        public MccDaq.MccBoard DaqBoard;
        public MccDaq.ErrorInfo ULStat;
        public MccDaq.Range Range;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //First Lets make sure there's a USB-2408 of some kind plugged in,
            //then we'll check to see if it has analog outputs.
            System.Int16 BoardNum;
            System.Boolean Boardfound = false;
            for (BoardNum = 0; BoardNum < 99; BoardNum++)
            {

                DaqBoard = new MccDaq.MccBoard(BoardNum);
                if (DaqBoard.BoardName.Contains("2408-2AO"))
                {
                    Boardfound = true;
                    this.Text = "2408-2AO found as board number: " + BoardNum.ToString();
                    ULStat = DaqBoard.FlashLED();
                    nudChannelNumber.Maximum = 1;
                    return;
                }

                if (DaqBoard.BoardName.Contains("2408"))
                {
                    Boardfound = true;
                    this.Text = "USB-2408 found as board number: " + BoardNum.ToString();
                    System.Windows.Forms.MessageBox.Show("The USB-2408 you have does not have analog output capabilities.  You need a model with the -2AO suffix.  "
                   + "This program will now terminate.", "Wrong model USB-2408");
                    this.Close();
                    break;
                }
            }

            if (Boardfound == false)
            {
                System.Windows.Forms.MessageBox.Show("No USB-2408 series board found in system.  Please run InstaCal.", "No Board detected");
                this.Close();
            }
        }

        private void btnEnd_Click(object sender, EventArgs e)
        {
            //Return the DACs to their initial output values of 0VDC on exit
            ULStat = DaqBoard.VOut(Convert.ToInt16(nudChannelNumber.Value), MccDaq.Range.Bip10Volts, 0, MccDaq.VOutOptions.Default);
            ULStat = DaqBoard.VOut(Convert.ToInt16(nudChannelNumber.Value), MccDaq.Range.Bip10Volts, 1, MccDaq.VOutOptions.Default);
            this.Close();
        }

        private void btnDacOut_Click(object sender, EventArgs e)
        {
            //Output 1 value to a DAC using the AOut() method
            ushort ushCountOut;

            ULStat = DaqBoard.FromEngUnits(MccDaq.Range.Bip10Volts, Convert.ToSingle(txtVoltsOut.Text),out ushCountOut);

            ULStat = DaqBoard.AOut(Convert.ToInt16(nudChannelNumber.Value), MccDaq.Range.Bip10Volts, ushCountOut);
            lblStatus.Text = "Status  " + ULStat.Message;
        }

        private void btnVOut_Click(object sender, EventArgs e)
        {
            //Output 1 value to a DAC using the VOut() method
            ULStat = DaqBoard.VOut(Convert.ToInt16(nudChannelNumber.Value), MccDaq.Range.Bip10Volts, 
            Convert.ToSingle(txtVoltsOut.Text), MccDaq.VOutOptions.Default);
            lblStatus.Text = "Status  " + ULStat.Message;
        }
    }
}