using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MccDaq;

namespace AnalogOutScan
{
    public partial class Form1 : Form
    {
        public MccDaq.MccBoard DaqBoard;
        public MccDaq.ErrorInfo ULStat;
        MccDaq.Range AORange;
		System.IntPtr Mhandle ;
		System.Int32 NumPoints ;
		System.Int32 FirstPoint;
		System.Int32 ChannelCount = 1;  

        BERGtools.Oscilloscope scope;
        BERGtools.FunctionGenerator FuncGen;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //First Lets make sure there's a USB-2408 of some kind plugged in,
            //then we'll check to see if it has analog outputs.
            System.Int16 BoardNum;
            System.Boolean Boardfound = false;
            for (BoardNum = 0; BoardNum < 99; BoardNum++)
            {

                DaqBoard = new MccDaq.MccBoard(BoardNum);
                if (DaqBoard.BoardName.Contains("2408-2AO"))
                {
                    Boardfound = true;
                    this.Text = "2408-2AO found as board number: " + BoardNum.ToString();
                    ULStat = DaqBoard.FlashLED();
                    nudChannelNumber.Maximum = 1;
                    break;
                }

                if (DaqBoard.BoardName.Contains("2408"))
                {
                    Boardfound = true;
                    this.Text = "USB-2408 found as board number: " + BoardNum.ToString();
                    System.Windows.Forms.MessageBox.Show("The USB-2408 you have does not have analog output capabilities.  You need a model with the -2AO suffix.  "
                   + "This program will now terminate.", "Wrong model USB-2408");
                    this.Close();
                    break;
                }
            }

            if (Boardfound == false)
            {
                System.Windows.Forms.MessageBox.Show("No USB-2408 series board found in system.  Please run InstaCal.", "No Board detected");
                this.Close();
            }

            scope = new BERGtools.Oscilloscope();
            scope.Initialize(pbScope);                      //select the picturebox to use as stripchart
            scope.TraceColor(0,Color.White);                  //select the color of the data trace for channel 0
            scope.TraceColor(1,Color.Blue);                   //select the color of the data trace for channel 1
            scope.FontColor(Color.Beige);  //select the color of the text
            scope.BackgroundColor(Color.DarkGreen);       //select the background color
            scope.ScaleColor(Color.DarkKhaki);               //select scale and tick mark color
            scope.YAxisScale(BERGtools.YAxisScaleVal.Bipolar);  //select if scale is uni or bi polar
            scope.YScale(-10, 10, 8);                       //enter the min, max, and number of ticks for the Y Scale
            scope.DrawBackground();                         //draw the bitmap into the picture box as background

            FuncGen = new BERGtools.FunctionGenerator();
            
        }

        private void btnEnd_Click(object sender, EventArgs e)
        {
            System.Single VOutVolts;

			 AORange = MccDaq.Range.Bip10Volts;
			 VOutVolts = 0;

             ULStat = DaqBoard.VOut(Convert.ToInt16(nudChannelNumber.Value), AORange, VOutVolts, MccDaq.VOutOptions.Default);
			 ULStat = DaqBoard.StopBackground(MccDaq.FunctionType.AoFunction);
			 this.Close();
        }

        public void errhandler(MccDaq.ErrorInfo ULStat)
        {
            //Generic UL error handler
            System.Windows.Forms.MessageBox.Show(ULStat.Message, "Universal Library Error ");
            lblMode.Text = "Idle";
            btnStartStop.Text = "Start";
            ULStat = DaqBoard.StopBackground(MccDaq.FunctionType.AoFunction);
            ULStat = MccDaq.MccService.WinBufFreeEx(Mhandle); // Free up memory for use by other programs
        }

        private void btnStartStop_Click(object sender, EventArgs e)
        {
            if (btnStartStop.Text == "Start") 
            {
				btnStartStop.Text = "Stop";
				this.Refresh();
				RunDacs();
			}
			 else
            {
				btnStartStop.Text = "Start";
				tmrAOut.Enabled = false;
				ULStat = DaqBoard.StopBackground(MccDaq.FunctionType.AoFunction);
				if (ULStat.Value !=  MccDaq.ErrorInfo.ErrorCode.NoErrors) this.Close();
				lblMode.Text = "Idle";
				ULStat = MccDaq.MccService.WinBufFreeEx(Mhandle); // Free up memory for use by other programs
				if (ULStat.Value != MccDaq.ErrorInfo.ErrorCode.NoErrors) this.Close();
			}
        }

        public void RunDacs()
        {
			System.Int32 i;
            Single MaxDacValue, DutyCycle;
            Single DCOffset = 0;
			System.Int32 AORate; 
            System.Int32 intCurrentChannel = 0;
            System.Int32 intPointsPerWaveform;
            System.Int32 OutputChannel;

            FuncGen.Initialize(1);   //enter number of channels.

			/*'Here we are generating the type of waveform to output. 
			'you can skip down to the line of ''''''''''' for the syntax 
			'that controls the scan*/
			try
            {
                Single.TryParse(txtAmplitude.Text, out MaxDacValue);
                FuncGen.amplitude(MaxDacValue, intCurrentChannel);
			}
			catch (Exception esc)
            {
				System.Windows.Forms.MessageBox.Show(txtAmplitude.Text + " is not a valid Amplitude", "Invalid Amplitude ");
            btnStartStop.Text = "Start";
			return;
			}

			try{
                Single.TryParse(txtOffset.Text, out DCOffset);
                FuncGen.offset(DCOffset, intCurrentChannel);
			}
			catch (Exception esc)
            {
				System.Windows.Forms.MessageBox.Show(txtOffset.Text + " is not a valid Offset", "Invalid Offset ");
				btnStartStop.Text = "Start";
				return;
			}

            try
            {
                Single.TryParse(txtDutyCycle.Text, out DutyCycle);
                FuncGen.DutyCycle(DutyCycle, intCurrentChannel);
            }
            catch (Exception esc)
            {
                System.Windows.Forms.MessageBox.Show(txtDutyCycle.Text + " is not a valid Duty Cycle (set in %, example: 50", "Invalid Duty Cycle ");
                btnStartStop.Text = "Start";
                return;
            }

			try
            {
                Int32.TryParse(txtAORate.Text,out AORate);
			}
			catch (Exception esc)
            {
			   System.Windows.Forms.MessageBox.Show(txtAORate.Text + " is not a valid Rate", "Invalid Rate ");
				btnStartStop.Text = "Start";
				return;
			}


            try     //number of point per period
            {    
                Int32.TryParse(txtPointsPerWaveform.Text,out  intPointsPerWaveform);
                FuncGen.PointsPerPeriod(intPointsPerWaveform, intCurrentChannel);
                NumPoints = intPointsPerWaveform;
			}
			catch (Exception esc)
            {
			   System.Windows.Forms.MessageBox.Show(txtPointsPerWaveform.Text + " is not a valid value", "Invalid number of points per waveform period.");
				btnStartStop.Text = "Start";
				this.tmrAOut.Enabled = false;
				return;
			}


            float[,] ADDataV;
            ADDataV = new float[ChannelCount , NumPoints];
            System.Int16[] DataArray;
            DataArray = new System.Int16[ChannelCount * NumPoints];
			for( i=0; i<NumPoints; i++)
            {
				ADDataV[0,i] = (float) 0;
				DataArray[i] =  0; 
			}

            OutputChannel = Convert.ToInt32(nudChannelNumber.Value);
             
          //select a predetermined wavetype
            if (rbNoise.Checked)
                FuncGen.WaveSelect(BERGtools.WaveType.Noise, intCurrentChannel, 1);
            else  if  (rbRampWave.Checked)
                FuncGen.WaveSelect(BERGtools.WaveType.Ramp, intCurrentChannel, 1);
            else  if (rbSineWave.Checked)
                FuncGen.WaveSelect(BERGtools.WaveType.Sine, intCurrentChannel, 1);
            else  if (rbSquareWave.Checked)
                FuncGen.WaveSelect(BERGtools.WaveType.Square, intCurrentChannel, 1);
            else  if (rbTriangleWave.Checked)
                FuncGen.WaveSelect(BERGtools.WaveType.Triangle, intCurrentChannel, 1);

             //FuncGen.CreateWave(out ADDataV);   //create the waveform using FunctionGenerator class
             ADDataV = FuncGen.CreateWave();

            for ( i = 0; i < NumPoints; i++)     //convert to counts from volts
                ULStat = DaqBoard.FromEngUnits(MccDaq.Range.Bip10Volts, ADDataV[0, i],out  DataArray[i]);


            scope.UpdateScope(ref ADDataV);  //display 1 period of the waveform to be sent out.


            //'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
            //'Look here to see how to Load the array, set options and start the scan
			Mhandle = MccDaq.MccService.WinBufAllocEx(NumPoints); // set aside memory to hold data
            if (Mhandle == IntPtr.Zero) return;

            FirstPoint = 0;
            lblMode.Text = "Running";


            //'move the array out to computer memory
            ULStat = MccDaq.MccService.WinArrayToBuf( DataArray, Mhandle, FirstPoint, NumPoints);

            //'Set scan options
			MccDaq.ScanOptions AOOptions ;   //Default
            AOOptions = MccDaq.ScanOptions.Background;
			if (chkContinuous.Checked)  
				AOOptions = AOOptions |(MccDaq.ScanOptions.Continuous);

            ULStat = DaqBoard.AOutScan(Convert.ToInt16(nudChannelNumber.Value), Convert.ToInt16(nudChannelNumber.Value), 
				NumPoints,ref AORate, MccDaq.Range.Bip10Volts, Mhandle, AOOptions);
            if (ULStat.Value != MccDaq.ErrorInfo.ErrorCode.NoErrors) {
					errhandler(ULStat);
					return;
			}

            //'once the scan is running, check the timer for status.
            tmrAOut.Enabled = true;
    }

        private void tmrAOut_Tick(object sender, EventArgs e)
        {
            short Status;
            System.Int32 CurCount, CurIndex;

            ULStat = DaqBoard.GetStatus(out Status, out CurCount, out CurIndex, FunctionType.AoFunction);
            if (ULStat.Value != MccDaq.ErrorInfo.ErrorCode.NoErrors)
            {
                errhandler(ULStat);
                return;

            }
            if (Status == 1) 
            {
                this.lblMode.Text = "Running";
            }
            else{
                this.lblMode.Text = "Idle";
            }
            lblCurIndex.Text = CurIndex.ToString();
        }

    }
}