﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using MccDaq;
using BERGtools;

namespace Temperature_InSW_Timed
{
    public partial class Form1 : Form
    {
        public MccDaq.MccBoard DaqBoard;
        public MccDaq.ErrorInfo ULStat;
        public MccDaq.Range Range;
        public System.Int32 numchannels = 0;

        BERGtools.Thermometer TempMeter;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //First Lets make sure there's a USB-2408 of some kind plugged in,
            //then we'll check to see if it has analog outputs.
            System.Int16 BoardNum;
            System.Boolean Boardfound = false;
            for (BoardNum = 0; BoardNum < 99; BoardNum++)
            {
                DaqBoard = new MccDaq.MccBoard(BoardNum);
                if (DaqBoard.BoardName.Contains("2408-2AO"))
                {
                    Boardfound = true;
                    this.Text = "2408-2AO found as board number: " + BoardNum.ToString();
                    ULStat = DaqBoard.FlashLED();
                    break;
                }

                if (DaqBoard.BoardName.Contains("2408"))
                {
                    Boardfound = true;
                    this.Text = "USB-2408 found as board number: " + BoardNum.ToString();
                    DaqBoard.FlashLED();
                    break;
                }
            }

            if (Boardfound == false)
            {
                System.Windows.Forms.MessageBox.Show("No USB-2408 series board found in system.  Please run InstaCal.", "No Board detected");
                this.Close();
            }

            //Initialize objects on the form needing attention
            cmboAInRange.Items.Add("Celcius");
            cmboAInRange.Items.Add("Fahrenheit");
            cmboAInRange.Items.Add("Kelvin");
            cmboAInRange.Items.Add("NoScale");
            cmboAInRange.SelectedIndex = 0;

            cmboOptions.Items.Add("Filter");
            cmboOptions.Items.Add("NoFilter");
            cmboOptions.SelectedIndex = 0;

            //Assume channels are configured differential ended inputs annd for Temperature
            nudAInChannel.Maximum = 8;

            //Initialize the thermometer
            TempMeter = new BERGtools.Thermometer();
            TempMeter.Initialize(pbTMeter);              //select the picturebox to use as stripchart
            TempMeter.TemperatureColor(Color.Red);      //select the color of the data trace for channel 1
            TempMeter.FontColor(Color.Black);            //select the color of the text
            System.Drawing.Color mycolor = this.BackColor;   //Me.PictureBox1.BackColor
            TempMeter.BackgroundColor(mycolor);
            TempMeter.ScaleColor(Color.Black);       //select scale and tick mark color
            TempMeter.YScale(0, 100, 10);                //enter the min, max, and number of ticks for the Y Scale
            TempMeter.DrawBackground();                  //draw the bitmap into the picture box as background

            ////set up sample timing
            //for (int i = 1; i < 11; i++)
            //    cbRate.Items.Add(i);
            //cbRate.SelectedIndex = 9;
        }

        private MccDaq.TempScale SelectRange(System.Int32 ComboBoxIndex ) 
        {

            MccDaq.TempScale TmpScale = MccDaq.TempScale.Celsius;
            switch (ComboBoxIndex)
            {
            case 0:
                    TmpScale =  MccDaq.TempScale.Celsius;
                    TempMeter.YScale(0, 100, 10);
                    TempMeter.DrawBackground();
                    break;
            case 1:
                    TmpScale =  MccDaq.TempScale.Fahrenheit;
                    TempMeter.YScale(32, 100, 19);
                    TempMeter.DrawBackground();
                    break;
            case 2:
                    TmpScale = MccDaq.TempScale.Kelvin;
                    TempMeter.YScale(250, 350, 10);
                    TempMeter.DrawBackground();
                    break;
            case 3:
                    TmpScale =  MccDaq.TempScale.NoScale;
                    TempMeter.YScale(-1, 1, 2);
                    TempMeter.DrawBackground();
                    break;
            }

            return TmpScale;

        }

        public MccDaq.ThermocoupleOptions SelectOptions(System.Int32 ComboBoxIndex) 
        {

            MccDaq.ThermocoupleOptions TCOptions = ThermocoupleOptions.Filter;
            switch (ComboBoxIndex)
            {
                case 0:
                    TCOptions =  MccDaq.ThermocoupleOptions.Filter;
                    break;
                case 1:
                    TCOptions =  MccDaq.ThermocoupleOptions.NoFilter;
                    break;
            }

            return TCOptions;

        }

        private void btnGetOneDataPointUsingTIn_Click(object sender, EventArgs e)
        {
            System.Single dataval;

            MccDaq.TempScale TmpScale = SelectRange(cmboAInRange.SelectedIndex);
            MccDaq.ThermocoupleOptions TCOptions = SelectOptions(cmboOptions.SelectedIndex);

            ULStat = DaqBoard.TIn(Convert.ToInt16(nudAInChannel.Value), TmpScale,out dataval, TCOptions);
            lblTInStatus.Text = ULStat.Message;
            lblEngUnits.Text = dataval.ToString();

            TempMeter.updateTMeter(ref dataval);
        }

        private void Button6_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}
